from typing import List, Tuple

import pymysql
from sqlalchemy import create_engine
from enums.DatabaseDialectEnum import DatabaseDialectEnum
from enums.DatabaseTypeEnum import DatabaseTypeEnum
from components.Model.InsumoModel import InsumoModel, InsumoModelOracle
from components.Databases.Enums.MysqlDataTypeEnum import MysqlDataTypeEnum
from components.Databases.Enums.MysqlDataTypeORMEnum import MysqlDataTypeORMEnum


from sqlalchemy import Table, Column, MetaData
import logging

logger = logging.getLogger()


class Mysql:

    def __init__(self, host: str, port: int, user: str, password: str, database: str) -> None:
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.engine = None
        self.connection = None

    def get_basic_connection(self):
        try:
            if isinstance(self.connection, type(None)):
                self.connection = pymysql.connect(host=self.host, port=self.port, user=self.user,
                                                  password=self.password, db=self.database)
        except Exception as e:
            logger.error(f"Error obteniendo conexion básica de Oracle. {e}")
        finally:
            return self.connection

    def create_engine(self) -> None:
        try:
            dialect = DatabaseDialectEnum.MYSQL.value
            url = f"{dialect}://{self.user}:{self.password}@{self.host}:{str(self.port)}/{self.database}?charset=utf8mb4"
            self.engine = create_engine(url)
        except Exception as e:
            logger.error(f"Error creando engine de Mysql. {e}")

    def create_model(self, tablename: str, fields: List[Tuple[str]], db_target: str, modelName: str = "TableModel"):
        model = None
        try:
            db_args = {
                '__tablename__': tablename,
                '__table_args__': {'extend_existing': True}
            }
            if db_target == DatabaseTypeEnum.ORACLE.value:
                model = type(modelName, (InsumoModelOracle,), db_args)
            else:
                model = type(modelName, (InsumoModel,), db_args)
            for field in fields:
                logger.debug(f"Attribute: {field}")
                name = field[0]
                if field[2] != -1:
                    size = int(field[2] / 4)
                    try:
                        if not isinstance(field[3], type(None)) and field[3] > 0:
                            precision, scale = field[2], field[3]
                            if scale > precision:
                                precision = field[3]
                                scale = field[2]
                            data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value(precision=precision, scale=scale)
                        else:
                            data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value(size)
                    except TypeError:
                        data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value()
                else:
                    data_type = MysqlDataTypeORMEnum[MysqlDataTypeEnum(field[1]).name].value()
                setattr(model, name, Column(data_type))
            model = model.__table__
        except Exception as e:
            logger.error(f"Error creando modelo dinámico en Mysql con nombre {tablename}. {e}")
        finally:
            return model

