from typing import List, Tuple

import psycopg2
from sqlalchemy import create_engine
from components.Model.InsumoModel import InsumoModel, InsumoModelOracle
from enums.DatabaseDialectEnum import DatabaseDialectEnum
from enums.DatabaseTypeEnum import DatabaseTypeEnum
from components.Databases.Enums.PostgresDataTypeEnum import PostgresDataTypeEnum
from components.Databases.Enums.PostgresDataTypeORMEnum import PostgresDataTypeORMEnum

import logging
from sqlalchemy import Table, Column, MetaData

logger = logging.getLogger()


class Postgres:

    def __init__(self, host: str, port: int, user: str, password: str, database: str, schema: str) -> None:
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.schema = schema
        self.engine = None
        self.DEFAULT_VAR_LENGHT = 100
        self.connection = None

    def get_basic_connection(self):
        try:
            if isinstance(self.connection, type(None)):
                self.connection = psycopg2.connect(host=self.host, port=self.port, user=self.user,
                                                   password=self.password, database=self.database,
                                                   options="-c search_path="+self.schema)
        except Exception as e:
            logger.error(f"Error obteniendo conexion básica de Oracle. {e}")
        finally:
            return self.connection

    def create_engine(self) -> None:
        try:
            dialect = DatabaseDialectEnum.POSTGRES.value
            url = f"{dialect}://{self.user}:{self.password}@{self.host}:{str(self.port)}/{self.database}" \
                  f"?options=-csearch_path={self.schema}"
            self.engine = create_engine(url)
        except Exception as e:
            logger.error(f"Error creando engine de Postgres. {e}")

    def create_model(self, tablename: str, fields: List[Tuple[str]], db_target: str, modelName: str = "TableModel"):
        model = None
        try:
            db_args = {
                '__tablename__': tablename,
                '__table_args__': {'extend_existing': True}
            }
            if db_target == DatabaseTypeEnum.ORACLE.value:
                model = type(modelName, (InsumoModelOracle,), db_args)
            else:
                model = type(modelName, (InsumoModel,), db_args)
            for field in fields:
                logger.debug(f"Attribute: {field}")
                name = field[0]
                if field[2] != -1:
                    try:
                        if not isinstance(field[3], type(None)) and field[3] > 0:
                            precision, scale = field[2], field[3]
                            if scale > precision:
                                precision = field[3]
                                scale = field[2]
                            data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value(precision=precision, scale=scale)
                        else:
                            data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value(field[2])
                    except TypeError:
                        data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value()
                else:
                    data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value(self.DEFAULT_VAR_LENGHT)
                setattr(model, name, Column(data_type))
            model = model.__table__
        except Exception as e:
            logger.error(f"Error creando modelo dinámico en Postgres con nombre {tablename}. {type(e)}. {e}")
        finally:
            return model
